// WAPI.LIB
// Biblioteca de funes para uso da API do Windows
// ------------------------------------------------
//                    Janeiro/2007, Paulo Buzinello
//                         http://buzinello.com/prg
//                              paulo@buzinello.com
//                                      Londrina/PR
// ------------------------------------------------
//
#include "macros.ch"
#include "wapi.h"


//**************************************************************************************************
function ReadWReg(cKeyPath,; // caminho da chave, semelhante  rvore de diretrios
                  cEntry   ; // nome da entrada cujo dado ser lido (* para ler todas)
                  )          // --------------------------------------------------------------------
                             // OBS1: O retorno ser uma matriz, que conter uma matriz para cada um
                             //       dos tens lidos. Estas contero apenas trs elementos: tipo de
                             //       dado (B, C, N ou T), o nome da entrada e o contedo desta.
                             // OBS2: Os dados retornados so de 4 tipos: binrio (B), numrico (N),
                             //       caractere (C) ou texto (T). Este ltimo poder ser um conjunto
                             //       de vrias strings ASCIIZ, que no Registry so separadas por um
                             //       byte 0. O final deste conjunto  marcado por um byte 0 extra.
                             // OBS3: Ser retornada uma matriz, que estar vazia, caso ocorra algum
                             //       erro que impossibilite a leitura. No sucesso, a matriz conter
                             //       em cada elemento uma matriz com os dados das entradas lidas. A
                             //       matriz de uma entrada conter 3 elementos: tipo, nome e dados.
                             //       Os dois elementos iniciais sero sempre strings. O ltimo ser
                             //       uma string (tipo C), numrico (tipo N), uma matriz (tipo B) ou
                             //       texto (tipo T) com cada linha separada pelo par CR/LF. No caso
                             //       do dado ser do tipo binrio, sua matriz conter os valores dos
                             //       bytes lidos, sempre limitados entre 0 e 255.

local aRet  := {}
local cFile := UniqFName(WAPITmpDir())
local nLine := 0
local cBuff
local cName
local cType
local xData
local aData
local l

WAPIError(_kERROR_NONE)

RunWAPICmd("-READREGISTRY:"+Quote(cKeyPath)+_kARGS_SEP+;
                            Quote(cEntry  )+_kARGS_SEP+;
                            Quote(cFile   )            )

if ReadRetFile(cFile,@cBuff)
   while _kTRUE
      cLine := Left(cBuff,At(_kEOL,cBuff)-1)
      cBuff := SubStr(cBuff,Len(cLine)+2+1)
      //
      // A primeira linha sempre conter o cdigo de retorno.
      if ++nLine = 1
         if Val(cLine) != _kERROR_NONE
            WAPIError(Val(cLine))
            return aRet
         end
         loop
      end

      if (i := At("=",cLine)) = 0
         exit
      end

      cName :=   Left(cLine,i-1  )
      cType := SubStr(cLine,i+1,1)
      xData := SubStr(cLine,i+3  )
      
      if     cType == "B"
             // Para efeito visual, dados binrios sero delimitados por #, e os valores sero
             // finalidados com um ponto.
             xData := StrTran(xData,"#")
             aData := {}
             while (i := At(".",xData)) > 0
                AAdd(aData,Val(Left(xData,i-1)))
                xData := SubStr(xData,i+1)
             end
             AAdd(aRet,{cType,cName,aData})
             *
      elseif cType == "N"
             AAdd(aRet,{cType,cName,Val(xData)})
             *
      elseif cType == "C"
             AAdd(aRet,{cType,cName,xData})
             *
      elseif cType == "T"
             AAdd(aRet,{cType,cName,StrTran(xData,_kEOT,_kEOL)})
      end
   end
   ASort(aRet,,,{|x,y|x[2] < y[2]})
end
return aRet
